/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "Rect.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomRect.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomRect.h"
#include "gdom/PrivateIntGdomRect.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::Rect* obj)
{
    return WebCore::cacheDOMObject<WebCore::Rect, GDOMRect>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMRect::GDOMRect(PassRefPtr<WebCore::Rect> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMRect::~GDOMRect()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Rect.h"
#include <gdom/GdomRect.h>
#include <gdom/PrivateGdomRect.h>
#include <gdom/PrivateIntGdomRect.h>
#include <gdom/GdomRectprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomRect, gdom_rect, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomRectPrivate {
    class WebCore::Rect *item;
};

#define GDOM_RECT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_RECT, GdomRectPrivate))
*/
G_END_DECLS
WebCore::Rect *
gdom_rect_get_private(GdomRect *request)
{
    GDOMRect *res;
    res = static_cast<GDOMRect*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Rect*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_TOP,
    PROP_RIGHT,
    PROP_BOTTOM,
    PROP_LEFT,
};


static void gdom_rect_finalize(GObject* object)
{
    /*GdomRect* request = GDOM_RECT(object);*/
    /*GdomRectPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_rect_parent_class)->finalize(object);
}

static void gdom_rect__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomRect *self = GDOM_RECT(object);
    WebCore::Rect * gself = (gdom_rect_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_rect__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomRect *self = GDOM_RECT(object);
    WebCore::Rect * gself = (gdom_rect_get_private(self));
    switch (prop_id) {
    case PROP_TOP:
    {
         PassRefPtr<WebCore::CSSPrimitiveValue> ptr = gself->top();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_RIGHT:
    {
         PassRefPtr<WebCore::CSSPrimitiveValue> ptr = gself->right();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_BOTTOM:
    {
         PassRefPtr<WebCore::CSSPrimitiveValue> ptr = gself->bottom();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_LEFT:
    {
         PassRefPtr<WebCore::CSSPrimitiveValue> ptr = gself->left();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_rect_class_init(GdomRectClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_rect_finalize;
    g_klass->set_property = gdom_rect__set_property_fn;
    g_klass->get_property = gdom_rect__get_property_fn;

    g_object_class_install_property (g_klass, PROP_TOP,
            g_param_spec_object ("top", /* name */
                "rect_top", /* short description */
                "read-only  GdomCSSPrimitiveValue * Rect.top", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_PRIMITIVE_VALUE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_RIGHT,
            g_param_spec_object ("right", /* name */
                "rect_right", /* short description */
                "read-only  GdomCSSPrimitiveValue * Rect.right", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_PRIMITIVE_VALUE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_BOTTOM,
            g_param_spec_object ("bottom", /* name */
                "rect_bottom", /* short description */
                "read-only  GdomCSSPrimitiveValue * Rect.bottom", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_PRIMITIVE_VALUE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LEFT,
            g_param_spec_object ("left", /* name */
                "rect_left", /* short description */
                "read-only  GdomCSSPrimitiveValue * Rect.left", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_PRIMITIVE_VALUE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomRectPrivate));*/
}

static void gdom_rect_init(GdomRect* request)
{
    /*GdomRectPrivate* priv = GDOM_RECT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

